<?php
// $Id: ejournal_shortly.inc,v 1.1.4.2 2006/06/19 16:56:54 romca Exp $

/*
    ejournal.module by rca, ideas taken from Open Source Journal, and
    ejournal.module - Part of the ejournal module for Drupal
    Copyright (c) 2005  Sheldon Rampton <sheldon@prwatch.org>
                                                                                
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Ejournal shortly gives your editors chance to create/edit and attach 
    documents to your existing issues, *even if* the issue was already 
    published. This node is used mainly for short messages, notifications 
    and similar. The nodes are always visible and may be edited by 
    "ejournal editors".
*/

/** 
 * @file ejournal.module
 * @note See the module help for usage and administration information.
 */

function ejournal_shortly_menu($may_cache) {
  $items = array();
  if ($may_cache) {
    // public stuff
    $items[] = array('path' => 'ejournal_shortly',
    	'title' => t('Journal short messages archive'),
	'access' => user_access('access content'),
	'callback' => 'ejournal_shortly_public_page',
	'type' => MENU_SUGGESTED_ITEM);
  } 
  return $items;
} 

/**
 * hook_perm: Define user permissions for module ejournal
 */
function ejournal_shortly_perm() {
  $perms = array('ejournal shortly');
  $journals = db_query("SELECT * FROM {ejournal_journals} WHERE jid>=1");
  while($journal = db_fetch_object($journals)) {  
    $perms[] = $journal->jid .' ejournal shortly';
  }
  return $perms;
}

/**
 * Implementation of hook_block().
 */
function ejournal_shortly_block($op = 'list', $delta = 0) {
  if ($op == 'list') {
    $blocks[0]['info'] = t('Ejournal short messages');
    return $blocks;
  }
  else if ($op == 'view' && user_access('access content')) {
    switch($delta) {
      case 0:
	  $block['subject'] = t('Shortly');
	  $block['content'] = ejournal_shortly_list('block');
          return $block;
        break;
    }
  }
  
}


/**
 * Implementation of hook_nodeapi().
 */
function ejournal_shortly_nodeapi(&$node, $op, $teaser = NULL, $page = NULL) {
  
  //test if this node-type is considered to be an shortly-type was made from ejournal module
  //calling 'from_journal'function, that function should cache the results so we will only
  //pick them 
  $test = ejournal_shortly_from_journal(&$node);
  $jid = $test['shortly'];
  
  
  switch ($op) {
    case 'settings':
      break;          
      
    case 'insert':
	// insert shortly
	ejournal_shortly_add($jid, $node);
      break;
      
    case 'update':
      $update['iid'] = $node->nid_issue;
      ejournal_shortly_update($node->nid, $update);    
      // reinsert access rights for author
      ejournal_grant_article_author($node->nid, $node->uid);
      break;
      
    case 'delete':      
      ejournal_shortly_delete($node->nid);
      break;
            
    case 'validate':      
      if(isset($node->nid_issue) && !$node->nid_issue) {
	form_set_error('nid_issue', t('No issue selected for this %type.', array('%type' => 'message')));
      }
      if(!user_access('chief editor')) {
	$issue_status = db_result(db_query("SELECT ei.status FROM {ejournal_shortly} ea INNER JOIN {ejournal_issues} ei ON ea.iid=ei.iid AND ea.nid='%d'", $node->nid));
      
	//$issue_status may be empty when ISSUE_WORKING or article is going to be inserted
	if (!($issue_status == ISSUE_FUTURE || $issue_status == ISSUE_NOT_PUBLISHED)) {
	  //form_set_error('path', t('This article belongs to already published issue. Only chief editor may change its contents.'));
	  drupal_set_message(t('This article belongs to already published issue. Only chief editor may change its contents.'));
	  drupal_goto('node/'.$node->nid);
	}      
      }
      break;
      
  }
}

function ejournal_shortly_form_alter(&$form, $form_id) {

      // display issue and section form
      // if loaded for the first time, we will take the iid from the dtb (if saved already)
      if(!$node->nid_issue) {
	$shortly = ejournal_shortly_get_data($node->nid);
	$node->nid_issue = $shortly->iid;
      }
      $form['ejournal_shortly'] = array(
        '#type' => 'fieldset',
        '#weight' => 1,
        'form' => ejournal_article_form_edit_pre($form['#node'], $jid)
      );
}

function ejournal_shortly_from_journal($node) {
  static $shortly = array();
  
  if(array_key_exists($node->nid, $shortly)) {
    if($shortly[$node->nid]) {
      return array('shortly' => $shortly[$node->nid]);
    }
    else {
      return;
    }
  }
  // user may have changed journal-node_type relation after some nodes were already created  
  $jid = db_result(db_query("SELECT jid FROM {ejournal_shortly} WHERE nid='%s'", $node->nid));
  if($jid) {
    $shortly[$node->nid] = $jid;
    return array('shortly' => $shortly[$node->nid]);
  }
  
  $jid = db_result(db_query("SELECT jid FROM {ejournal_store} WHERE jvalue='%s' AND  jkey='shortly_node_types'", $node->type));
  
  $shortly[$node->nid] = $jid;
  if($jid) {
    return array('shortly' => $shortly[$node->nid]);
  }
}


function ejournal_shortly_get_data($nid) {    
  return db_fetch_object(db_query("SELECT * FROM {ejournal_shortly} WHERE nid='%s'", $nid));
}

/** 
 * ejournal_shortly_add: add a node to an issue
 *
 * @param $iid
 *   An issue ID
 * @param $nid
 *   A node ID
 */
function ejournal_shortly_add($jid, $node) {
        
  db_query("INSERT INTO {ejournal_shortly} (jid, nid, iid) VALUES ('%d', '%d', '%d')", $jid, $node->nid, $node->nid_issue);
  
  ejournal_grant_article_inserted($node->nid);
  
}

function ejournal_shortly_update($nid, $arg = array() ) {

  foreach($arg as $key => $value) {
    if(isset($value)) {    
      $sql[] = "$key='" . db_escape_string($value) . "' ";
    }
  }
  if(!$sql) {
    return;
  }
  $set = implode(', ', $sql);

  return db_query("UPDATE {ejournal_shortly} SET $set WHERE nid='%d'", $nid);

}

/** 
 * ejournal_shortly_delete: delete a shortly from an issue
 *
 * @param $nid
 *   A node ID
 */
function ejournal_shortly_delete($nid) {
  db_query("DELETE FROM {ejournal_shortly} WHERE nid = '%d'", $nid);
  ejournal_grant_article_deleted($nid);
}

/** 
 * ejournal_shortly_journal_form: adds a form element to ejournal
 *
 * @param $edit
 *   An array with form values (if any)
 * @param $nodetypes
 *   An array of nodetypes that were not registered yet (are free for use)
 */

function ejournal_shortly_journal_form(&$edit, $journal, &$nodetypes) {
  
  $shortly_nodetypes = explode(',', $edit['shortly_node_types']);
  
  $form['shortly_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('E-journal shortly nodetypes'),
    '#description' => t('Here you can select node types reserved for shortly messages. These will be not as restricted as regular articles, may be edited even if the issue were published, but only editors and chief editors can create them. Node types already registered with (another) journal can not be selected.'),
    '#value' => $shortly_nodetypes,
    '#options' => $nodetypes
  );

  foreach($shortly_nodetypes as $type) {
    unset($nodetypes[$type]);
  }
  return $form;
}

/**
 * Implementation of hook_help().
 */
function ejournal_shortly_help($section = 'admin/help#ejournal_shortly') {
  switch ($section) {
    case 'admin/modules#description':
      $output = t('Allows specific users to add short messages to the ejournal issues.');
      break;
    case 'admin/settings/ejournal_shortly':
      $output = t('<p>Settings for extra ejournal module (adding short messages to issues).</p>');
      break;
    case 'admin/help#ejournal_shortly':
      $output = t("<p>This module helps add shortly message to ejournal issues.</p>");
      break;
  }
  return $output;
}


function ejournal_shortly_list($type = 'block') {
  
  $arg0 = arg(0);
  if(is_numeric(arg(1))) {
    $and_clause = " AND n.nid != '".db_escape_string(arg(1))."' ";
  }

  if($arg0 == 'node') {
    $jid = db_result(db_query("SELECT jid FROM {ejournal_articles} WHERE nid='%d'", arg(1)));
  }
  else if ($arg0 == 'ejournal') {
    $jid = db_result(db_query("SELECT jid FROM {ejournal_articles} WHERE jid='%d'", arg(1)));
  }
  
  if($jid) {
    $sql = db_rewrite_sql("SELECT n.nid, n.title, n.changed FROM {node} n INNER JOIN {ejournal_shortly} es ON n.nid = es.nid AND es.jid='$jid' " . $and_clause . " ORDER BY n.created DESC");
    //print_r($sql);
    
    $result = db_query_range($sql, 0, variable_get('ejournal_shortly_num', '5'));
    
    if (db_num_rows($result)) {
      $content = node_title_list($result);
    }
    $content .= l(t('Archive...'), "ejournal_shortly/$jid"); 
    return $content;
  }
}

/** 
 * ejournal_shortly_public_page: Display the archive of the short messages for journal(s).
 */
function ejournal_shortly_public_page() {
  $content = ejournal_shortly_archive(NULL, NULL, NULL);
  print theme('ejournal_page', "<div class=\"ejournal-shortly\">$content</div>\n");
}

function ejournal_shortly_archive($jid=NULL, $iid=NULL, $type = NULL, $limit = 20) {

  $shortly = ejournal_shortly_get($jid, $iid, $type, $limit);
  
  if($shortly) {
      
    foreach($shortly as $node) {
      $items[] = '[' . format_date($node->changed) . '] ' . l($node->title, 'node/'. $node->nid);
    }    
    $content = theme('node_list', $items);
    $content .= theme('pager', NULL, $limit);    
  }
  return $content;
  
}

function ejournal_shortly_get($jid=NULL, $iid=NULL, $type=NULL, $limit=20, $full=FALSE) {

  if(arg(0) == 'node') {
    $nid_exclude = " AND n.nid != '".db_escape_string(arg(1))."' ";
  }
  else {
    $jid = $jid ? $jid : arg(1);
    $iid = $iid ? $iid : arg(2);
  }
  
  if ($jid && !is_numeric($jid)) {
    $journal = ejournal_journal_get($jid);
    $jid = $journal->jid;
  }
  
  if(is_numeric($iid)) {
    $and_clause = " AND es.iid = '".db_escape_string($iid)."' ";
  }  
  
  if($type) {
    $type_clause = " AND n.type IN ('".db_escape_string($type)."') ";
  }
  
  $sql = db_rewrite_sql("SELECT n.nid, n.title, n.changed FROM {node} n INNER JOIN {ejournal_shortly} es ON n.nid = es.nid WHERE es.jid = '".db_escape_string($jid)."' $and_clause  $type_clause $nid_exclude ORDER BY n.created DESC");  
  
  $sql_count = "SELECT count(es.nid) FROM {ejournal_shortly} es INNER JOIN {node} n ON es.nid=n.nid WHERE es.jid='".db_escape_string($jid)."' $and_clause $type_clause $nid_exclude";
  
  $result = pager_query($sql, $limit, 0, $sql_count);

  while($node = db_fetch_object($result)) {
    if($full) {
      $items[] = node_load(array('nid' => $node->nid));
    }
    else {
      $items[] = $node;
    }
  }

  return $items;  
}
?>